<?php
if (!defined('ABSPATH')) exit;

class WPSUB_Login {
    public function __construct() {
        add_action('wp_enqueue_scripts', [$this, 'enqueue_scripts']);
        add_action('wp_ajax_nopriv_wpsub_ajax_login', [$this, 'ajax_login']);
        add_shortcode('wpsub_login_form', [$this, 'render_login_form']);
    }

    // Enqueue the custom login script
    public function enqueue_scripts() {
        wp_enqueue_script('wpsub-login-js', WPSUB_PLUGIN_URL . 'assets/js/wpsub-login.js', ['jquery'], time(), true);
        wp_localize_script('wpsub-login-js', 'wpsub_login_obj', [
            'ajax_url' => esc_url(admin_url('admin-ajax.php')),
            'login_nonce' => wp_create_nonce('wpsub_login_nonce'),
        ]);
    }

    // Handle the AJAX login
    public function ajax_login() {
        // Verify nonce
        if (
            !isset($_POST['nonce']) || 
            !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['nonce'] ?? '')), 'wpsub_login_nonce')
        ) {
            wp_send_json_error(['message' => 'Security check failed.']);
        }

        // Validate and sanitize inputs
        $username = sanitize_text_field(wp_unslash($_POST['username'] ?? ''));
        $password = sanitize_text_field(wp_unslash($_POST['password'] ?? ''));
        $remember = !empty($_POST['rememberme']) ? true : false;

        if (empty($username) || empty($password)) {
            wp_send_json_error(['message' => 'Username and password are required.']);
        }

        // Attempt login using WordPress authentication
        $creds = [
            'user_login'    => $username,
            'user_password' => $password,
            'remember'      => $remember,
        ];

        $user = wp_signon($creds, is_ssl());

        if (is_wp_error($user)) {
            $error_message = $user->get_error_message();
            if (strpos($error_message, 'The password you entered for the username') !== false) {
                // Translators: %s is replaced with the username.
                $custom_message = sprintf(__('The password you entered for the username <strong>%s</strong> is incorrect.', 'wizweb-post-submission'),
                    $username
                ) . ' <a href="' . esc_url(home_url('/forgot-password')) . '">' . 
                    __('Lost Your Password?', 'wizweb-post-submission') . 
                    '</a>';

            } 
            else if (strpos($error_message, 'The password you entered for the email') !== false) {

                // Translators: %s is replaced with the email address entered by the user.
                $custom_message = sprintf(__('The password you entered for the email address <strong>%s</strong> is incorrect.', 'wizweb-post-submission'),
                    $username
                ) . ' <a href="' . esc_url(home_url('/forgot-password')) . '">' . 
                    __('Lost Your Password?', 'wizweb-post-submission') . 
                    '</a>';

            }
            else {
                $custom_message = $error_message ? wp_kses($error_message, ['a' => ['href' => [], 'title' => []]]) : __('An error occurred. Please try again.', 'wizweb-post-submission');
            }
        
            wp_send_json_error(['message' => wp_kses_post($custom_message)]);
        } else {
            // Login successful: Redirect user
            wp_send_json_success(['redirect_url' => esc_url(home_url('/dashboard'))]);
        }
        

        wp_die();
    }

    public static function render_login_form() {
        if (is_user_logged_in()) {
            return '<p>' . esc_html__('You are already logged in.', 'wizweb-post-submission') . 
                ' <a href="' . esc_url(home_url('/dashboard')) . '">' . esc_html__('Go to Dashboard', 'wizweb-post-submission') . '</a>.</p>';
        }

        ob_start();
        include plugin_dir_path(__FILE__) . '../../templates/auth/login-form.php';
        return ob_get_clean();
    }

    public static function is_registration_enabled() {
        return get_option('users_can_register') == 1;
    }
}
