<?php
if (!defined('ABSPATH')) exit;

class WPSUB_Signup {

    public function __construct() {
        add_action('wp_enqueue_scripts', [$this, 'enqueue_scripts']);
        add_action('wp_ajax_nopriv_wpsub_ajax_signup', [$this, 'ajax_signup']);
        add_shortcode('wpsub_signup_form', [$this, 'render_signup_form']);
    }

    // Enqueue the custom signup script
    public function enqueue_scripts() {
        wp_enqueue_script('wpsub-signup-js', WPSUB_PLUGIN_URL . 'assets/js/wpsub-signup.js', ['jquery'], time(), true);
        wp_localize_script('wpsub-signup-js', 'wpsub_signup_obj', [
            'ajax_url' => esc_url(admin_url('admin-ajax.php')),
        ]);
    }

    // Handle the AJAX signup
    public function ajax_signup() {

        // Verify the nonce for security
        if (
            !isset($_POST['nonce']) || 
            !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['nonce'])), 'wpsub_signup_nonce')
        ) {
            wp_send_json_error(['message' => 'Invalid form submission. Please try again.']);
        }

        // Sanitize inputs
        $username = sanitize_text_field(wp_unslash($_POST['wpsub_username'] ?? ''));
        $email = sanitize_email(wp_unslash($_POST['wpsub_email'] ?? ''));
        $password = sanitize_text_field(wp_unslash($_POST['wpsub_password'] ?? ''));

        // Validate input
        if (empty($username) || empty($email) || empty($password)) {
            wp_send_json_error(['message' => 'All fields are required.']);
        }

        if (!is_email($email)) {
            wp_send_json_error(['message' => 'Please provide a valid email address.']);
        }

        if (username_exists($username)) {
            wp_send_json_error(['message' => 'Username is unavailable. Please try another one.']);
        }

        if (email_exists($email)) {
            wp_send_json_error(['message' => 'An account already exists with this email address.']);
        }

        // Create a new user with 'subscriber' role
        $user_id = wp_create_user($username, $password, $email);
        if (is_wp_error($user_id)) {
            wp_send_json_error(['message' => 'Could not complete signup: ' . esc_html($user_id->get_error_message())]);
        }

        // Set default role
        $user = new WP_User($user_id);
        $user->set_role('subscriber');

        // Send signup notifications
        wpsub_notify_on_signup($user_id);

        // Success response
        wp_send_json_success([
            'message' => 'Signup successful! You can now <a href="' . esc_url(site_url('/login')) . '">login</a>.'
        ]);

        wp_die();
    }

    public static function render_signup_form() {
        if ( is_user_logged_in() ) {
            return '<p>' . esc_html__('You are already logged in.', 'wizweb-post-submission') . '</p>';
        }

        ob_start();
        include plugin_dir_path( __FILE__ ) . '../../templates/auth/signup-form.php';
        return ob_get_clean();
    }
}
