<?php

class WPSUB_Payment_Handler {

    public function __construct() {
        add_shortcode('wpsub_payment_page', [$this, 'render_payment_button']);
        add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_assets' ] );
        add_action( 'wp_ajax_submit_pay_post', [ $this, 'payment_success_handler' ] );
    }

    public function enqueue_assets() {
        
        // Get PayPal settings
        $payments_settings = get_option('wpsub_payments_setting', []);
        if ($payments_settings['paypal_status']) {
            wp_enqueue_script(
                'paypal-sdk',
                'https://www.paypal.com/sdk/js?client-id=' . $payments_settings["paypal_client_id"],
                [],
                null,
                true
            );            
        }
    
        // Enqueue the custom payment script with a version based on the plugin version or timestamp
        $script_version = filemtime( WPSUB_PLUGIN_URL . 'assets/js/wpsub-payments.js' );
        wp_enqueue_script(
            'wpsub-payments-script',
            WPSUB_PLUGIN_URL . 'assets/js/wpsub-payments.js',
            ['jquery'],
            $script_version,
            true
        );
    }

    public function render_payment_button() 
    {
        if ( ! is_user_logged_in() ) {
            return "<p>You are not logged in. Please log in to continue. <a href=".esc_url( home_url( '/login' ) ).">Login</a></p>";
            exit;
        }
        
        // Check if 'post_id' and '_wpnonce' are set in the URL
        if (isset($_GET['post_id']) && isset($_GET['_wpnonce'])) {
            // Unsanitize the nonce input (remove slashes first)
            $nonce = wp_unslash($_GET['_wpnonce']);
    
            // Sanitize nonce for security
            $nonce = sanitize_text_field($nonce); // Now sanitize
    
            // Verify the nonce
            if (!wp_verify_nonce($nonce, 'payment_button_nonce')) {
                return '<p>Nonce verification failed. Please try again.</p>';
            }
        }
    
        if (isset($_GET['post_id'])) {
            // Sanitize and retrieve the post ID
            $post_id = intval($_GET['post_id']);
    
            // Get post details
            $post = get_post($post_id);
            if (!$post || $post->post_type !== 'post') {
                return '<p>Invalid post ID or the post does not exist.</p>';
            }
    
            // Prepare post details for the template
            $post_details = [
                'ID'          => $post->ID,
                'title'       => get_the_title($post),
                'date'        => get_the_date('', $post),
                'status'      => $post->post_status,
                'permalink'   => get_permalink($post),
            ];
    
            $payments_settings = get_option('wpsub_payments_setting', []);
    
            // Pass details to the template
            ob_start();
            include WPSUB_PLUGIN_DIR . 'templates/posts/payments.php';
            return ob_get_clean();
        }
    
        return '<p>Post ID is not defined.</p>';
    }
    

    public function payment_success_handler() 
    {
        // Ensure the request is valid and secure
        if (!is_user_logged_in() || !check_ajax_referer('wpsub_nonce', 'security', false)) {
            wp_send_json_error(['message' => 'Unauthorized access.']);
        }

        // Check if the request is via AJAX
        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            wp_send_json_error(['message' => 'Invalid request.']);
        }

        // Validate required data
        if (!isset($_POST['post_id'], $_POST['transaction_id'])) {
            wp_send_json_error(['message' => 'Missing required data.']);
        }

        // Sanitize input data
        $post_id        = intval($_POST['post_id']);
        $transaction_id = sanitize_text_field(wp_unslash($_POST['transaction_id']));
        $amount         = isset($_POST['amount']) ? sanitize_text_field(wp_unslash($_POST['amount'])) : '';
        $gateway        = isset($_POST['gateway']) ? sanitize_text_field(wp_unslash($_POST['gateway'])) : '';

        // Validate post ID
        if (get_post_type($post_id) !== 'post') {
            wp_send_json_error(['message' => 'Invalid post ID.']);
        }

        // Update post status to 'publish'
        $updated = wp_update_post([
            'ID'          => $post_id,
            'post_status' => 'publish',
        ]);

        if (is_wp_error($updated)) {
            wp_send_json_error(['message' => 'Failed to update post status.']);
        }

        // Save transaction details
        global $wpdb;
        // Safely create the table name
        $table_name = $wpdb->prefix . 'wpsub_transactions';

        $inserted = $wpdb->insert(// phpcs:ignore WordPress.DB.DirectDatabaseQuery
            $table_name,
            [
                'user_id'        => get_current_user_id(),
                'post_id'        => intval($post_id),
                'transaction_id' => sanitize_text_field($transaction_id),
                'gateway'        => sanitize_text_field($gateway),
                'amount'         => floatval($amount),
                'status'         => 'Completed',
            ],
            ['%d', '%d', '%s', '%s', '%f', '%s'] // Define formats for data types
        );

        if ($inserted === false) {
            wp_send_json_error(['message' => 'Failed to save transaction details.']);
        }

        $last_inserted_id = $wpdb->insert_id;

        // Retrieve the details of the inserted row
        $txn_details = $wpdb->get_row(
            $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $last_inserted_id), 
        );

        // Send Email
        wpsub_payment_successful_email_to_user($txn_details);

        wpsub_payment_successful_email_to_admin($txn_details);

        wpsub_article_publish_email_to_user($txn_details);

        // Return success response
        wp_send_json_success([
            'message'      => get_option('wpsub_paid_success_message', 'Your post has been published successfully.'),
            'redirect_url' => get_permalink($post_id)        
        ]);
    }

}

new WPSUB_Payment_Handler();
