<?php
class WPSUB_Post_Submission {
    
    public function __construct() {
        add_shortcode( 'wpsub_post_submission_page', [ $this, 'render_post_form' ] );
        add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_assets' ] );
        add_action( 'wp_ajax_submit_post', [ $this, 'handle_post_submission' ] );
        add_action( 'wp_ajax_nopriv_submit_post', [ $this, 'redirect_to_login' ] );        
        $this->hide_toolbar_for_users(); 
    }

    public function enqueue_assets() 
    {
        // CSS
        wp_enqueue_style( 'wpsub-style', WPSUB_PLUGIN_URL . 'assets/css/wpsub.css', true, "1.0.4");        
        wp_enqueue_style('select2-css', WPSUB_PLUGIN_URL .'assets/css/select2.min.css', null, true);
        wp_enqueue_style('tagify-css', WPSUB_PLUGIN_URL .'assets/css/tagify.css', null, true);

        // Javascript
        wp_enqueue_script('select2', WPSUB_PLUGIN_URL . 'assets/js/select2.min.js', array('jquery'), true, true);
        wp_enqueue_script('multi-tag', WPSUB_PLUGIN_URL .'assets/js/multi-tag.js', array('jquery'), true, true);
        wp_enqueue_script( 'wpsub-script', WPSUB_PLUGIN_URL . 'assets/js/wpsub-script.js', [ 'jquery' ], '1.0.2', true );

        // Payment Settings
        $payments_settings = get_option( 'wpsub_payments_setting', [] );

        // Captcha
        $recaptcha_keys = get_option( 'wpsub_recaptcha_keys', [] );
        $recaptcha_status = isset($recaptcha_keys['status']) ? $recaptcha_keys['status'] : '';
        wp_localize_script( 'wpsub-script', 'wpsub_settings', [ 
            'recaptcha_status'      => $recaptcha_status,
            'wpsub_tags'            => get_option('wpsub_show_tags'),
            'redirect_url'          => get_option( 'wpsub_redirect_url'),
            'ajax_url'              => admin_url( 'admin-ajax.php' ),
            'price'                 => isset($payments_settings['price']) ? $payments_settings['price'] : '5'
        ] );

         // Pass existing tags to JavaScript
        $all_tags = get_tags(array('hide_empty' => false));
        $tags_list = array_map(function($tag) {
            return $tag->name;
        }, $all_tags);

        wp_localize_script('tagify-js', 'fspTags', $tags_list);

        // Include TinyMCE editor scripts.
        wp_enqueue_editor();
        // wp_enqueue_script( 'wpsub-tinymce-init', WPSUB_PLUGIN_URL . 'assets/js/tinymce-init.js', [ 'jquery', 'wp-tinymce' ], true, true );

    }

    public function render_post_form() 
    {
        if ( ! is_user_logged_in() ) {
            return "<p>You are not logged in. Please log in to continue. <a href=".esc_url( home_url( '/login' ) ).">Login</a></p>";
            exit;
        }

        ob_start();
        include WPSUB_PLUGIN_DIR . 'templates/posts/submission-form.php';
        return ob_get_clean();
    }

    public function sanitize_file_name($filename) {
        // Remove any path information
        $filename = basename($filename);

        // Replace spaces with underscores
        $filename = str_replace(' ', '_', $filename);

        // Remove unwanted characters
        $filename = preg_replace('/[^A-Za-z0-9_\-\.]/', '', $filename);

        // Limit the file name length
        $filename = substr($filename, 0, 255);

        return $filename;
    }

    public function handle_post_submission() {

        if ( ! is_user_logged_in() || ! check_ajax_referer( 'wpsub_nonce', 'security' ) ) {
            wp_send_json_error( 'Unauthorized submission.' );
        }
    
        // Check if the form is submitted via AJAX.
        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            wp_send_json_error('Invalid request.');
        }
    
        $errors = [];
    
        // Post Input Value
        $title      = isset($_POST['title']) ? sanitize_text_field(wp_unslash($_POST['title'])) : '';
        $content    = isset($_POST['content']) ? wp_kses_post(wp_unslash($_POST['content'])) : '';
    
        if (get_option('wpsub_category_selection_mode') == "multiple_required") {
            $selected_categories = isset($_POST['categories']) ? array_map('intval', $_POST['categories']) : [];
        } else {
            $selected_categories = isset($_POST['category']) ? [intval($_POST['category'])] : [];
        }

        $tags_input = isset($_POST['tags']) ? wp_unslash($_POST['tags']) : '';
        $tags = !empty($tags_input) ? array_map('sanitize_text_field', explode(',', $tags_input)) : [];

        $featureImage = isset($_FILES['featured_image']['name']) ? sanitize_file_name($_FILES['featured_image']['name']) : '';

        if ($featureImage) {
            $allowed_file_types = ['image/jpeg', 'image/png', 'image/gif'];
            $file_type = mime_content_type($_FILES['featured_image']['tmp_name']);
            if (!in_array($file_type, $allowed_file_types)) {
                $errors['featured_image'] = 'Invalid file type. Only JPG, PNG, and GIF files are allowed.';
            }
            if ($_FILES['featured_image']['size'] > 2 * 1024 * 1024) { // 2MB limit
                $errors['featured_image'] = 'File size exceeds the maximum limit of 2MB.';
            }
        }

    
        // Validation for required fields
        if (empty($title)) {
            $errors['title'] = 'Title is required.';
        }
    
        if (empty($content)) {
            $errors['content'] = 'Content is required.';
        }
    
        if (get_option('wpsub_category_selection_mode') == "multiple_required" && empty($_POST['categories'])) {
            $errors['categories[]'] = 'Categories are required.';
        }
    
        if (get_option('wpsub_category_selection_mode') == "single_required" && empty($_POST['category'])) {
            $errors['category'] = 'Category is required.';
        }
    
        if (get_option('wpsub_show_tags') == "show" && count(array_filter($tags)) <= 0) {
            $errors['tags'] = 'Tags are required.';
        }
    
        if (get_option('wpsub_show_featured_image') == "show" && empty($featureImage)) {
            $errors['featured_image'] = 'Featured image is required.';
        }
    
        // Check if the title already exists
        if (get_option('wpsub_unique_title')) {
            $existing_post_query = new WP_Query([
                'post_type'   => 'post',
                'title'       => $title,
                'posts_per_page' => 1
            ]);
            if ($existing_post_query->have_posts()) {
                $errors['title'] = 'A post with this title already exists. Please choose a unique title.';
            }
        }
    
        // Return errors if any
        if (!empty($errors)) {
            wp_send_json_error(['errors' => $errors]);
        }
    
        $category_id = isset($_POST['category']) ? intval($_POST['category']) : 0;
        $tags = isset($_POST['tags']) ? sanitize_text_field(wp_unslash($_POST['tags'])) : '';
    
        // Handle the uploaded featured image
        if ( isset( $_FILES['featured_image'] ) && ! empty( $_FILES['featured_image']['name'] ) ) {
            $file_name = sanitize_file_name( wp_unslash( $_FILES['featured_image']['name'] ) );
            $file_type = wp_check_filetype( $file_name );
        
            // Check if the file type is allowed
            if ( in_array( $file_type['ext'], [ 'jpg', 'jpeg', 'png', 'gif' ], true ) ) {
                $uploaded_file = $_FILES['featured_image'];
                $upload_overrides = ['test_form' => false]; // Disable form check
                $movefile = wp_handle_upload( $uploaded_file, $upload_overrides );
        
                if ( $movefile && ! isset( $movefile['error'] ) ) {
                    // File upload successful
                    $attachment = [
                        'guid'           => $movefile['url'],
                        'post_mime_type' => $movefile['type'],
                        'post_title'     => sanitize_file_name( $uploaded_file['name'] ),
                        'post_content'   => '',
                        'post_status'    => 'inherit',
                    ];
        
                    $attachment_id = wp_insert_attachment( $attachment, $movefile['file'] );
                    require_once ABSPATH . 'wp-admin/includes/image.php';
                    $attachment_data = wp_generate_attachment_metadata( $attachment_id, $movefile['file'] );
                    wp_update_attachment_metadata( $attachment_id, $attachment_data );
        
                    // set_post_thumbnail( $post_id, $attachment_id );
                    $featured_image_id = $attachment_id;
                } else {
                    // Handle upload errors
                    $errors['featured_image'] = sprintf( 'Error uploading the image: %s', esc_html( $movefile['error'] ) );
                }
            } else {
                $errors['featured_image'] = 'Invalid file type. Only JPG, JPEG, PNG, and GIF files are allowed.';
            }
        }
        
        // Prepare post data and validate.
        $post_data = [
            'post_title'    => $title,
            'post_content'  => $content,
            'post_status'   => get_option('wpsub_default_post_status', 'pending'),
            'post_category' => $selected_categories,
            'post_author'   => get_current_user_id(),
        ];
    
        $post_id = wp_insert_post($post_data);
    
        if ($post_id) {
            // Assign featured image to the post
            if (!empty($featured_image_id)) {
                wp_update_post([
                    'ID' => $featured_image_id,
                    'post_parent' => $post_id,
                ]);
                set_post_thumbnail($post_id, $featured_image_id);
            }
    
            // Assign categories to the post
            if (!empty($selected_categories)) {
                wp_set_post_categories($post_id, $selected_categories);
                // Recalculate term counts for all assigned categories
                foreach ($selected_categories as $cat_id) {
                    wp_update_term_count_now([$cat_id], 'category');
                }
            }
    
            // Assign tags to the post
            if (!empty($tags)) {
                wp_set_post_tags($post_id, $tags);
            }

            // Send Email
            wpsub_notify_on_post_submission(get_current_user_id(), $post_id);


            // Post Paye Mode
            $submission_mode = isset($_POST['submission_mode']) ? sanitize_text_field(wp_unslash($_POST['submission_mode'])) : '';
            if($submission_mode === "paid") {
                
                $nonce = wp_create_nonce('payment_button_nonce');
                $url = add_query_arg(array(
                    'post_id'   => $post_id,
                    '_wpnonce'  => $nonce,
                ), site_url('/payments'));
                wp_send_json_success([
                    'message'           => get_option('wpsub_free_success_message', 'Your post has been submitted for review.'), 
                    'redirect_url'      => $url,
                    'submission_mode'   => 'paid'
                ]); 
                exit;
            }
            

            wp_send_json_success(['message' => get_option('wpsub_free_success_message', 'Your post has been submitted for review.')]);         
    
        } else {
            wp_send_json_error(['errors' => get_option('wpsub_error_message', 'An error occurred while submitting the post.')]);
        }
    }

    public function redirect_to_login() {
        wp_redirect( site_url( '/login' ) );
        exit;
    }

    public function hide_toolbar_for_users() {
        add_filter('show_admin_bar', function($show) {
            // Show toolbar only for administrators.
            if (current_user_can('administrator')) {
                return true;
            }
            return false;
        });
    }

}
